-- Tests for bbsmingensys.cpkg
Package $apcocoa/ts/bbsmingensys

Alias BBSGen := $apcocoa/bbsmingen;

Define Initialize()
EndDefine;



--Weight matrix
Define TestBBSGen01()
TestRing::=QQ[x[1..2]];
Using TestRing Do
  OO:=$apcocoa/borderbasis.Box([1,1]); 
  BO:=$apcocoa/borderbasis.Border(OO);   
  N:=Len(Indets());
  Result:=Wmat(OO,BO,N);
  Expected:=Mat([
  [0, 2, 1, 2, 0, 2, 1, 2, -1, 1, 0, 1, -1, 1, 0, 1],
  [2, 0, 2, 1, 1, -1, 1, 0, 2, 0, 2, 1, 1, -1, 1, 0]]);
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: WMat!");
EndUsing;

EndDefine;


--K[c]-linear summand of Trace Polynomial
Define TestBBSGen02();
R::=QQ[x[1..2]];
 Using R Do
 OO:=$apcocoa/borderbasis.Box([1,1]);
 BO:=$apcocoa/borderbasis.Border(OO);   
 Mu:=Len(OO);
 Nu:=Len(BO);
 N:=Len(Indets());
 Mon:=x[1]^2x[2];--------Target Monomial
 X:=x[1];   ------------Choice of the Indeterminate
 EndUsing;
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]]; 
Using XX Do
  Result:=BBSGen.TraceSyzLin(OO,BO,N); 
  Expected:= [ t[1,2,1,3] + t[1,2,2,4],2t[1,2,1,2] + 2t[1,2,3,4], t[1,2,1,3] + t[1,2,2,4],2t[1,2,1,4]];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error TrSyzLin!");
EndUsing;

EndDefine;



--K[c]-Linear Summand of the Trace Polynomial for a given degree and an indeterminate
Define TestBBSGen03();
 R::=QQ[x[1..2]];
 Using R Do
 OO:=$apcocoa/borderbasis.Box([1,1]);
 BO:=$apcocoa/borderbasis.Border(OO);   
 Mu:=Len(OO);
 Nu:=Len(BO);
 N:=Len(Indets());
 Mon:=x[1]^2x[2];--------Target Monomial
 X:=x[1];   ------------Choice of the Indeterminate
 EndUsing;
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]]; 
 Using XX Do 
     Result:= BBSGen.TraceSyzLinStep(Mon,X,OO,BO,N);
     Expected:=[t[1,2,1,3] + t[1,2,2,4]];
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: TrSyzLinStep!");
 EndUsing;

EndDefine;









--Trace Polynomial for a given degree and an indeterminate
Define TestBBSGen04();

 R::=QQ[x[1..2]];
 Using R Do
 OO:=$apcocoa/borderbasis.Box([1,1]);
 BO:=$apcocoa/borderbasis.Border(OO);   
 Mu:=Len(OO);
 Nu:=Len(BO);
 N:=Len(Indets());
 Mon:=x[1]^2x[2];--------Target Monomial
 X:=x[1];   ------------Choice of the Indeterminate
 EndUsing;
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]]; 
 Using XX Do
     Result:= BBSGen.TraceSyzStep(Mon,X,OO,BO,N);
     Expected:=c[1,2]t[1,2,3,1] + c[2,2]t[1,2,3,2] + c[3,2]t[1,2,3,3] + c[4,2]t[1,2,3,4] + c[1,4]t[1,2,4,1] + c[2,4]t[1,2,4,2] + c[3,4]t[1,2,4,3] + c[4,4]t[1,2,4,4] +
     t[1,2,1,3] + t[1,2,2,4];
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: TrSyzStep!");
 EndUsing;

EndDefine;




--Trace Polynomial for every non-standard degree of \tau d and every indeterminate
Define TestBBSGen05()

 R::=QQ[x[1..2]];
 Using R Do
 OO:=$apcocoa/borderbasis.Box([1,1]);
 BO:=$apcocoa/borderbasis.Border(OO);   
 Mu:=Len(OO);
 Nu:=Len(BO);
 N:=Len(Indets());
 Mon:=x[1]^2x[2];--------Target Monomial
 X:=x[1];   ------------Choice of the Indeterminate
 EndUsing;
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]]; 
 Using XX Do
     Result:= BBSGen.TraceSyzFull(OO,BO,N);
     Expected:=[c[1,2]t[1,2,3,1] + c[2,2]t[1,2,3,2] + c[3,2]t[1,2,3,3] + c[4,2]t[1,2,3,4] +
     c[1,4]t[1,2,4,1] + c[2,4]t[1,2,4,2] + c[3,4]t[1,2,4,3] + c[4,4]t[1,2,4,4] + 
     t[1,2,1,3] + t[1,2,2,4],  2c[1,1]t[1,2,2,1] + 2c[2,1]t[1,2,2,2] + 2c[3,1]t[1,2,2,3] + 2c[4,1]t[1,2,2,4]+ 
     2c[1,3]t[1,2,4,1] + 2c[2,3]t[1,2,4,2] + 2c[3,3]t[1,2,4,3] + 2c[4,3]t[1,2,4,4] 
     + 2t[1,2,1,2] + 2t[1,2,3,4],
     c[1,2]t[1,2,3,1] + c[2,2]t[1,2,3,2] + c[3,2]t[1,2,3,3] + c[4,2]t[1,2,3,4] + 
    c[1,4]t[1,2,4,1] + c[2,4]t[1,2,4,2] + c[3,4]t[1,2,4,3] + c[4,4]t[1,2,4,4] + 
    t[1,2,1,3] + t[1,2,2,4],
    2c[1,2]c[3,1]t[1,2,2,1] + 2c[1,4]c[4,1]t[1,2,2,1] + 2c[2,2]c[3,1]t[1,2,2,2] +
    2c[2,4]c[4,1]t[1,2,2,2] + 2c[3,1]c[3,2]t[1,2,2,3] + 2c[3,4]c[4,1]t[1,2,2,3] +
    2c[3,1]c[4,2]t[1,2,2,4] + 2c[4,1]c[4,4]t[1,2,2,4] + 2c[1,2]c[3,3]t[1,2,4,1] +
    2c[1,4]c[4,3]t[1,2,4,1] + 2c[2,2]c[3,3]t[1,2,4,2] + 2c[2,4]c[4,3]t[1,2,4,2] +
    2c[3,2]c[3,3]t[1,2,4,3] + 2c[3,4]c[4,3]t[1,2,4,3] + 2c[3,3]c[4,2]t[1,2,4,4] + 
    2c[4,3]c[4,4]t[1,2,4,4] + 2c[1,1]t[1,2,2,3] + 2c[2,1]t[1,2,2,4] + 2c[1,4]t[1,2,3,1] + 
    2c[2,4]t[1,2,3,2] + 2c[3,4]t[1,2,3,3] + 2c[4,4]t[1,2,3,4] + 2c[1,3]t[1,2,4,3] +
    2c[2,3]t[1,2,4,4] + 2t[1,2,1,4]];
     
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: TrSyzFull!");
 EndUsing;

EndDefine;




--(I,J) ^th Position of Jacobi Identity Matrix
Define TestGLPK06();
  
 R::=QQ[x[1..3]];
  
 Using R Do
    OO:=[1,x[1]];
    BO:=$apcocoa/borderbasis.Border(OO);   
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do
     Result:= BBSGen.JacobiStep(1,2,OO,BO,N);
     Expected:=[[[-c[1,3]t[1,2,1,1] + c[1,1]t[1,2,1,2] - c[2,3]t[1,2,1,2] +
     c[1,3]t[1,2,2,2] - c[1,4]t[1,3,1,1] + c[1,2]t[1,3,1,2] - 
     c[2,4]t[1,3,1,2] + c[1,4]t[1,3,2,2] - c[1,5]t[2,3,1,1] - 
     c[2,5]t[2,3,1,2] + c[1,5]t[2,3,2,2] ]]];
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: JacobiStep!");
 EndUsing;
EndDefine;






--K[c]-linear summands of the entries of Jacobi Identity Matrix
Define TestGLPK07();
  
 R::=QQ[x[1..3]];
  
 Using R Do
    OO:=[1,x[1]];
    BO:=$apcocoa/borderbasis.Border(OO);   
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do
     Result:= BBSGen.JacobiLin(OO,BO,N);
     Expected:=[[ [-t[2,3,1,2],0],[ t[2,3,1,1] - t[2,3,2,2], t[2,3,1,2]]]];
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: JacobiLin!");
 EndUsing;
EndDefine;



--All the entries of Jacobi Identity Matrix
Define TestGLPK08();
  
 R::=QQ[x[1..3]];
  
 Using R Do
    OO:=[1,x[1]];
    BO:=$apcocoa/borderbasis.Border(OO);   
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do 
     Result:= BBSGen.JacobiFull(OO,BO,N);
     Expected:=[[[-c[2,1]t[1,2,1,2] + c[1,3]t[1,2,2,1] - c[2,2]t[1,3,1,2] + c[1,4]t[1,3,2,1] + c[1,5]t[2,3,2,1] - t[2,3,1,2],
     -c[1,3]t[1,2,1,1] + c[1,1]t[1,2,1,2] - c[2,3]t[1,2,1,2] + c[1,3]t[1,2,2,2] - c[1,4]t[1,3,1,1] + c[1,2]t[1,3,1,2] - 
     c[2,4]t[1,3,1,2] + c[1,4]t[1,3,2,2] - c[1,5]t[2,3,1,1] - c[2,5]t[2,3,1,2] + c[1,5]t[2,3,2,2]],
     [c[2,1]t[1,2,1,1] - c[1,1]t[1,2,2,1] +c[2,3]t[1,2,2,1] - c[2,1]t[1,2,2,2] + c[2,2]t[1,3,1,1] - c[1,2]t[1,3,2,1] +
     c[2,4]t[1,3,2,1] - c[2,2]t[1,3,2,2] + c[2,5]t[2,3,2,1] + t[2,3,1,1] - t[2,3,2,2], 
     c[2,1]t[1,2,1,2] - c[1,3]t[1,2,2,1] + c[2,2]t[1,3,1,2] - c[1,4]t[1,3,2,1] - c[1,5]t[2,3,2,1] + t[2,3,1,2]]]];

     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: JacobiFull!");
 EndUsing;
EndDefine;


--Pure Power Indeterminates from K[c]
Define TestGLPK09();
   
  TestRing::=QQ[x,y];
  Using TestRing Do  
   OO:=$apcocoa/borderbasis.Box([1,1]);
   Result:= BBSGen.PurPow(OO);
   Expected:=[[1, 2], [2, 2], [3, 2], [4, 2], [1, 1], [2, 1], [3, 1], [4, 1]];
   $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: PurPow!");
 EndUsing;
EndDefine;


--Equivalance Classes wrt m^2
Define TestGLPK10();
   
  TestRing::=QQ[x,y];
  Using TestRing Do  
   OO:=$apcocoa/borderbasis.Box([1,1]);
   Result:= BBSGen.LinIndepGen(OO); 
   Expected:=[[[3, 3], [1, 1]], [[1, 2], [2, 4]], [[4, 3], [2, 1]], [[2, 2]], [[3, 1]], [[4, 4], [3, 2]], [4, 2], [4, 1]];
   $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: LinIndep!");
 EndUsing;
EndDefine;



--Arrow Degree of a given homogenous Poly from K[c]
Define TestGLPK11();

 R::=QQ[x[1..2]];
  
 Using R Do   
    OO:=$apcocoa/borderbasis.Box([1,1]);
    BO:=$apcocoa/borderbasis.Border(OO);
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
    W:=BBSGen.Wmat(OO,BO,N);
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do  
     F:= c[2,4]c[4,1] - c[3,3]c[4,2] - c[2,3] + c[3,4]; 
     Result:= BBSGen.Poldeg(F,OO,BO,N,W);
     Expected:=R :: Vector(1, 1) ;
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: Poldeg");
 EndUsing;
EndDefine;




--Non-Trivial Polynomials
Define TestGLPK12();

 R::=QQ[x[1..2]];
  
 Using R Do   
    OO:=$apcocoa/borderbasis.Box([1,1]);
    BO:=$apcocoa/borderbasis.Border(OO);
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
    W:=BBSGen.Wmat(OO,BO,N);
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do  
     Result:=BBSGen.NonTriv(OO,BO,W,N);
     Expected:=[
  [
    t[1,2,1,2],
    c[1,2]c[3,1] + c[1,4]c[4,1] - c[1,3],
    R :: Vector(1, 2)],
  [
    t[1,2,1,3],
    -c[1,1]c[2,2] - c[1,3]c[4,2] + c[1,4],
    R :: Vector(2, 1)],
  [
    t[1,2,1,4],
    -c[1,1]c[2,4] + c[1,2]c[3,3] + c[1,4]c[4,3] - c[1,3]c[4,4],
    R :: Vector(2, 2)],
  [
    t[1,2,2,2],
    c[2,2]c[3,1] + c[2,4]c[4,1] - c[2,3],
    R :: Vector(1, 1)],
  [
    t[1,2,2,3],
    -c[2,1]c[2,2] - c[2,3]c[4,2] - c[1,2] + c[2,4],
    R :: Vector(2, 0)],
  [
    t[1,2,2,4],
    -c[2,1]c[2,4] + c[2,2]c[3,3] + c[2,4]c[4,3] - c[2,3]c[4,4] - c[1,4],
    R :: Vector(2, 1)],
  [
    t[1,2,3,2],
    c[3,1]c[3,2] + c[3,4]c[4,1] + c[1,1] - c[3,3],
    R :: Vector(0, 2)],
  [
    t[1,2,3,3],
    -c[2,2]c[3,1] - c[3,3]c[4,2] + c[3,4],
    R :: Vector(1, 1)],
  [
    t[1,2,3,4],
    -c[2,4]c[3,1] + c[3,2]c[3,3] + c[3,4]c[4,3] - c[3,3]c[4,4] + c[1,3],
    R :: Vector(1, 2)],
  [
    t[1,2,4,2],
    c[3,1]c[4,2] + c[4,1]c[4,4] + c[2,1] - c[4,3],
    R :: Vector(0, 1)],
  [
    t[1,2,4,3],
    -c[2,2]c[4,1] - c[4,2]c[4,3] - c[3,2] + c[4,4],
    R :: Vector(1, 0)],
  [
    t[1,2,4,4],
    -c[2,4]c[4,1] + c[3,3]c[4,2] + c[2,3] - c[3,4],
    R :: Vector(1, 1)]];
     
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: NonTriv");
 EndUsing;
EndDefine;



--Non-Standard Polynomials from the Generating Set of I(B_O) wrt. Arrow Grading
Define TestGLPK13();

 R::=QQ[x[1..2]];
  
 Using R Do   
    OO:=$apcocoa/borderbasis.Box([1,1]);
    BO:=$apcocoa/borderbasis.Border(OO);
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
    W:=BBSGen.Wmat(OO,BO,N);
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do  
     Result:= BBSGen.NonStandPoly(OO,BO,W,N);
     Expected:=[ [ c[1,2]c[3,1] + c[1,4]c[4,1] - c[1,3],
    R :: Vector(1, 2)],
  [ c[1,1]c[2,2] + c[1,3]c[4,2] - c[1,4],
    R :: Vector(2, 1)],
  [ c[1,1]c[2,4] - c[1,2]c[3,3] - c[1,4]c[4,3] + c[1,3]c[4,4],
    R :: Vector(2, 2)],
  [c[2,2]c[3,1] + c[2,4]c[4,1] - c[2,3],
    R :: Vector(1, 1)],
  [c[2,1]c[2,4] - c[2,2]c[3,3] - c[2,4]c[4,3] + c[2,3]c[4,4] + c[1,4],
    R :: Vector(2, 1)],
  [c[2,2]c[3,1] + c[3,3]c[4,2] - c[3,4],
    R :: Vector(1, 1)],
  [c[2,4]c[3,1] - c[3,2]c[3,3] - c[3,4]c[4,3] + c[3,3]c[4,4] - c[1,3],
    R :: Vector(1, 2)],
  [c[2,4]c[4,1] - c[3,3]c[4,2] - c[2,3] + c[3,4],
    R :: Vector(1, 1)]];
     
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: NonStandPoly");
 EndUsing;
EndDefine;



----Non-Standard Indeterminates
Define TestGLPK14();

 R::=QQ[x[1..2]];
  
 Using R Do   
    OO:=$apcocoa/borderbasis.Box([1,1]);
    BO:=$apcocoa/borderbasis.Border(OO);
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
    W:=BBSGen.Wmat(OO,BO,N);
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do   
     Result:= BBSGen.NonStand(OO,BO,N,W);
     Expected:=[[c[1,3], R :: Vector(1, 2)], [c[1,4], R :: Vector(2, 1)], [c[2,3], R :: Vector(1, 1)], [c[3,4], R :: Vector(1, 1)]];
     
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: NonStand");
 EndUsing;
EndDefine;




----Finds the given list of polynomials and their arrow degrees
Define TestGLPK15();

 R::=QQ[x[1..2]];
  
 Using R Do   
    OO:=$apcocoa/borderbasis.Box([1,1]);
    BO:=$apcocoa/borderbasis.Border(OO);
    Mu:=Len(OO);
    Nu:=Len(BO);
    N:=Len(Indets()); 
    W:=BBSGen.Wmat(OO,BO,N);
 EndUsing;
 
 XX::=QQ[c[1..Mu,1..Nu],t[1..N,1..N,1..Mu,1..Mu]];  
 Using XX Do   
     Result:= BBSGen.BBFinder([t[1,2,3,4],t[1,2,2,4]],OO,BO,N,W);
     Expected:=[ [   [   R :: Vector(1, 2)],
     [t[1,2,3,4]],
     [ -c[2,4]c[3,1] + c[3,2]c[3,3] + c[3,4]c[4,3] - c[3,3]c[4,4] + c[1,3]]],
     [[   R :: Vector(2, 1)],
     [  t[1,2,2,4]],
     [ -c[2,1]c[2,4] + c[2,2]c[3,3] + c[2,4]c[4,3] - c[2,3]c[4,4] - c[1,4]]]];
     
     $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "Error: BBFinder");
 EndUsing;
EndDefine;



EndPackage;
