-- Tests for glpk.cpkg
Package $apcocoa/ts/glpk

Alias GLPK := $apcocoa/glpk;

Define Initialize()
EndDefine;



--LP-Problem with Simplex
Define TestGLPK01()

TestRing::=QQ[x[1..2]];

Using TestRing Do
  GL1 := 1/2 x[1] + x[2];
  GL2 := [];
  GL3 := [3/4 x[1] + x[2] -6];
  GL4 := [x[1]+x[2]-1];
  Bounds := [[1/3,6],[1,4]];
  Method := "Simplex";
  MinMax := "Max";
  Result:=GLPK.LPSolve(GL1, GL2, GL3, GL4, Bounds, Method, MinMax);
  Expected:=[x[1] - 266667/100000, x[2] - 4];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TODO: Some error message!");
EndUsing;

EndDefine;


-- Temporarily commented out. The Ubuntu virtual machines have GLPK 4.38
-- installed which contains a bug (see
-- http://www.mail-archive.com/bug-glpk@gnu.org/msg00342.html).
--
-- Uncomment as soon as the Ubuntu GLPK packages are updated.
--
--LP-Problem with Interior-points-method
--Define TestGLPK02()
--
--TestRing::=QQ[x[1..2]];
--
--Using TestRing Do
--  GL1 := 1/2 x[1] + x[2];
--  GL2 := [];
--  GL3 := [3/4 x[1] + x[2] -6];
--  GL4 := [x[1]+x[2]-1];
--  Bounds := [[1/3,6],[1,4]];
--  Method := "InterP";
--  MinMax := "Max";
--  Result:=GLPK.LPSolve(GL1, GL2, GL3, GL4, Bounds, Method, MinMax);
--  Expected:=[x[1] - 266667/100000, x[2] - 4];
--  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TODO: Some error message!");
--EndUsing;
--
--EndDefine;



--MIP-Problem with 1 integral variable
Define TestGLPK03()

TestRing::=QQ[x[1..2]];

Using TestRing Do
  GL1 := 1/2 x[1] + x[2];
  GL2 := [];
  GL3 := [3/4 x[1] + x[2] -6];
  GL4 := [x[1]+x[2]-1];
  Bounds := [[1/3,6],[1,4]];
  MinMax := "Max";
  Result:=GLPK.MIPSolve(GL1, GL2, GL3, GL4, Bounds, [x[1]], [], MinMax);
  Expected:=[x[1] - 3, x[2] - 15/4];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TODO: Some error message!");
EndUsing;

EndDefine;



--MIP-Problem with all variables integral
Define TestGLPK04()

TestRing::=QQ[x[1..2]];

Using TestRing Do
  GL1 := 1/2 x[1] + x[2];
  GL2 := [];
  GL3 := [3/4 x[1] + x[2] -6];
  GL4 := [x[1]+x[2]-1];
  Bounds := [[1/3,6],[1,4]];
  MinMax := "Max";
  Result:=GLPK.MIPSolve(GL1, GL2, GL3, GL4, Bounds, [x[1],x[2]], [], MinMax);
  Expected:=[x[1] - 2, x[2] - 4];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TODO: Some error message!");
EndUsing;

EndDefine;



--MIP-Problem with one variables binary
Define TestGLPK05()

TestRing::=QQ[x[1..2]];

Using TestRing Do
  GL1 := 1/2 x[1] + x[2];
  GL2 := [];
  GL3 := [3/4 x[1] + x[2] -6];
  GL4 := [x[1]+x[2]-1];
  Bounds := [[1/3,6],[1,4]];
  MinMax := "Max";
  Result:=GLPK.MIPSolve(GL1, GL2, GL3, GL4, Bounds, [], [x[1]], MinMax);
  Expected:=[x[1] - 1, x[2] - 4];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TODO: Some error message!");
EndUsing;

EndDefine;

Define TestGLPK06()

TestRing::=ZZ/(2)[x[1..4]];

Using TestRing Do
   F:=[
    x[1]x[2] + x[2]x[3] + x[2]x[4] + x[3]x[4] + x[1] + x[3] + 1, 
    x[1]x[2] + x[1]x[3] + x[1]x[4] + x[3]x[4] + x[2] + x[3] + 1, 
    x[1]x[2] + x[1]x[3] + x[2]x[3] + x[3]x[4] + x[1] + x[4] + 1, 
    x[1]x[3] + x[2]x[3] + x[1]x[4] + x[2]x[4] + 1
    ];
  CuttingNumber:=5;
  QStrategy:=1;
  CStrategy:=0;
  MinMax:="Max";

  Result:=GLPK.L01PSolve(F, CuttingNumber, QStrategy, CStrategy, MinMax);
  Expected:=[0, 1, 0, 1];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TestGLPK06 failed.");
EndUsing;

EndDefine;

Define TestGLPK07()

TestRing::=ZZ/(2)[x[1..4]];

Using TestRing Do
   F:=[
    x[1]x[2] + x[2]x[3] + x[2]x[4] + x[3]x[4] + x[1] + x[3] + 1, 
    x[1]x[2] + x[1]x[3] + x[1]x[4] + x[3]x[4] + x[2] + x[3] + 1, 
    x[1]x[2] + x[1]x[3] + x[2]x[3] + x[3]x[4] + x[1] + x[4] + 1, 
    x[1]x[3] + x[2]x[3] + x[1]x[4] + x[2]x[4] + 1
    ];
  
  Rule1:=0;
  Rule2:=0;
  MinMax:="Max";

  Result:=GLPK.RIPCSolve(F, Rule1, Rule2, MinMax);
  Expected:=[0, 1, 0, 1];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TestGLPK07 failed.");
EndUsing;

EndDefine;

Define TestGLPK08()

TestRing::=ZZ/(2)[x[1..4]];

Using TestRing Do
   F:=[
    x[1]x[2] + x[2]x[3] + x[2]x[4] + x[3]x[4] + x[1] + x[3] + 1, 
    x[1]x[2] + x[1]x[3] + x[1]x[4] + x[3]x[4] + x[2] + x[3] + 1, 
    x[1]x[2] + x[1]x[3] + x[2]x[3] + x[3]x[4] + x[1] + x[4] + 1, 
    x[1]x[3] + x[2]x[3] + x[1]x[4] + x[2]x[4] + 1
    ];
  
  Rule1:=0;
  Rule2:=0;
  MinMax:="Max";

  Result:=GLPK.RRPCSolve(F, Rule1, Rule2, MinMax);
  Expected:=[0, 1, 0, 1];
  $apcocoa/ts/assert.AssertEqualsExact(Result, Expected, "TestGLPK08 failed.");
EndUsing;

EndDefine;

EndPackage;
