-- This file is a template for an apcocoa test package.
-- It demonstrates all features of AUnit and you can derive your own tests
-- from it.
-- For best practices in unit testing we refer you to the jUnit FAQ:
--   http://junit.sourceforge.net/doc/faq/faq.htm

-- To see, how this template works, just enter
--   AUnit.RunTests("template");
-- in ApCoCoA.


-- Bundle your tests related to a particular CoCoA package inside a package 
-- within $apcocoa/ts.
Package $apcocoa/ts/template

  Alias Assert := $apcocoa/ts/assert;

-- We do not recommend to define an Initialize() function since it
-- has a special CoCoA semantic. Instead, use OneTimeSetUp(). 


-- The funcion OneTimeSetUp() - if present, it does not have to be -
-- is executed one time before the tests of this test package are executed.
Define OneTimeSetUp()
  PrintLn "OneTimeSetUp() called.";
EndDefine;


-- The funcion OneTimeTearDown() - if present, it does not have to be -
-- is executed one time after the tests of this test package are executed.
Define OneTimeTearDown()
  PrintLn "OneTimeTearDown called.";
EndDefine;


-- The function SetUp() - if present, it does not have to be - 
-- is executed one time before each test is executed.
Define SetUp()
  PrintLn "SetUp() called";
EndDefine;


-- The function TearDown() - if present, it does not have to be - 
-- is executed one time before each test is executed.
Define TearDown()
  PrintLn "TearDown() called";
EndDefine;


-- The function ExcludeTestsOn() - if present, it does not have to be - 
-- returns a list of pairs (each pair itself
-- a list with two string entries) which are test-platform combinations which
-- are excluded from testing. In particular, you can specify which tests
-- should not be executed on which platform.
--
--
-- Valid test identifiers are the names of the test functions (without the
-- parenthesis) of this test package or "*" for all test functions.
--
-- The valid platform identifiers are (see also $coclib.GetPlatformString()):
--  * Windows
--  * Linux-x86
--  * Linux-x86_64
--  * MacOSX 
--
Define ExcludeTestsOn()
  -- Exlude all tests on MacOSX and TestTemplate01 on Windows.
  Return [["*", "MacOSX"], ["TestTemplate01", "Windows"]];
EndDefine; -- ExcludeTestsOn


-- To write tests do the following.
--
-- (1) Define a function for every test you want to realize.
-- (2) Name the functions as follows: TestSomeFeatureOfMyPackage()
--     (you can find examples below or in the other already existing .ts packages)
-- (3) Every function should test only one feature of your package.
-- (4) A test can FAIL or SUCEED. The test fails iff the test functions
--     throws an error. To this end, use the AssertXY functions defined in
--     $apcocoa/ts/assert.cpkg as much as is possible.
--     Note, that your test functions do not return anything.
-- (5) If you miss some AssertXY function, add it to $apcocoa/ts/assert.cpkg.
-- (6) You are free to write an arbitrary number of helper functions inside
--     your test package, just make sure that their 
--     names DO NOT start with 'Test'.
-- (7) To run all the tests defined in the package, call
--       AUnit.RunTests("template");
-- 

Define TestTemplate01()

  TestRing::=QQ[x,y,z];

  Using TestRing Do

    Assert.AssertTrue(x=x, "x does not equal x.");

  EndUsing;

EndDefine;



Define TestTemplate02()

  TestRing::=ZZ/(2)[x[1..5]];

  Using TestRing Do

    -- implementation of the test

  EndUsing;

EndDefine;

-- TestTemplate03() and so on..

EndPackage;